#!/usr/bin/env python
import codecs
import re
import os
#from importlib_metadata import entry_points
from setuptools import setup, find_packages


with open("README.md", 'r', encoding="utf-8") as fh:
    long_description = fh.read()

def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()

requirements = ['paho-mqtt>=1.0', 'pyyaml>=6.0','schema>=0.7.2','netifaces>=0.10.0','jsonpath-ng>=1.5.2', 'pyuci']

setup(
    name='mqttled',
    version='0.0.5',
    description='MQTT control of OpenWRT LEDs with optional Home Assistant Discovery',
    author='Tom Grime',
    author_email='tom.grime@gmail.com',
    url='https://github.com/trevortrevor/mqttled',
    include_package_data=True,
    install_requires=requirements,
    license='MIT',
    zip_safe=False,
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=['OpenWRT, mqtt, Home Assistant'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: BSD",
    ],
    packages=find_packages(where='src'),
    package_dir={"": "src"},
    python_requires=">=3.9",
    entry_points={
        'console_scripts': [
            'mqttled=mqttled.__main__:run'
        ]
    }
)


#!/usr/bin/env python3

import os
import sys
import logging
import yaml
from schema import Schema, And, Or, Use, Optional, SchemaError
import jsonpath_ng
from uci import Uci

def json_path(txt):
    try:
        return jsonpath_ng.parse(txt)
    except Exception as e:
        raise SchemaError('Bad JsonPath format: %s' % txt)


def str_or_jsonPath(txt):
    if "$." in txt:
        return json_path(txt)
    return txt


def port_range(port):
    return 0 <= port <= 65535


schema = Schema({
    'mqtt': {
        'host': And(str, len),
        'port': And(int, port_range),
        'interface': And(str, len),
        Optional('username'): And(str, len),
        Optional('password'): And(str, len),
        Optional('cafile'): os.path.exists,
        Optional('certfile'): os.path.exists,
        Optional('keyfile'): os.path.exists,
        Optional('discovery'): And(str, len),
        Optional('basetopic'): And(str, len),
        Optional('subtopic'): And(str, len),
        Optional('model'): And(str, len)
    },
    'leds' : {
        Optional('all', default=True): (bool),
        Optional('include', default = []): (list),
        Optional('exclude', default = []): (list)
    },
    Optional('triggers'): (list),
    Optional("base64decode"): {
        'source': And(str, len, Use(str_or_jsonPath)),
        'target': And(str, len)
    },
})


def load_yaml(config_filename):
    with open(config_filename, 'r') as f:
        config = yaml.load(f, Loader=yaml.FullLoader)
        try:
            return schema.validate(config)
        except SchemaError as e:
            # Better error format
            error = str(e).splitlines()
            del error[1]
            raise Exception(' '.join(error))
        
def load_uci(configpath='mqttled'):
    u = Uci()
    try:
        config = u.get(configpath)
    except UciExceptionNotFound as e:
        error = str(e).splitlines()
        raise Exception(' '.join(error))
    return config
    